==> apache-camel-4.0.1-src.zip: apache-camel-4.0.1/core/camel-core/src/test/java/org/apache/camel/issues/OnExceptionBeforeErrorHandlerIssueTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.issues;

import org.apache.camel.ContextTestSupport;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;
import org.junit.jupiter.api.Test;

public class OnExceptionBeforeErrorHandlerIssueTest extends ContextTestSupport {

    @Test
    public void testOk() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(0);
        getMockEndpoint("mock:dead").expectedMessageCount(0);
        getMockEndpoint("mock:result").expectedMessageCount(1);

        template.sendBody("direct:start", "Hello World");

        assertMockEndpointsSatisfied();
    }

    @Test
    public void testKaboom() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(0);
        getMockEndpoint("mock:dead").expectedMessageCount(1);
        getMockEndpoint("mock:result").expectedMessageCount(0);

        template.sendBody("direct:start", "kaboom");

        assertMockEndpointsSatisfied();
    }

    @Test
    public void testIllegal() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(1);
        getMockEndpoint("mock:dead").expectedMessageCount(0);
        getMockEndpoint("mock:result").expectedMessageCount(0);

        template.sendBody("direct:start", "illegal");

        assertMockEndpointsSatisfied();
    }

    @Override
    protected RouteBuilder createRouteBuilder() throws Exception {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                onException(IllegalArgumentException.class).handled(true).setBody().constant("Handled").to("mock:error").end();

                // usually error handler should be defined first (before
                // onException),
                // but its not enforced
                errorHandler(deadLetterChannel("mock:dead").useOriginalMessage());

                from("direct:start").routeId("foo").noAutoStartup().process(new Processor() {
                    public void process(Exchange exchange) throws Exception {
                        String body = exchange.getIn().getBody(String.class);
                        if ("illegal".equals(body)) {
                            throw new IllegalArgumentException("I cannot do this");
                        } else if ("kaboom".equals(body)) {
                            throw new RuntimeException("Kaboom");
                        }
                    }
                }).to("mock:result");
            }
        };
    }
}

==> apache-camel-4.8.0-src.zip: apache-camel-4.8.0/core/camel-core/src/test/java/org/apache/camel/issues/OnExceptionBeforeErrorHandlerIssueTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.issues;

import org.apache.camel.ContextTestSupport;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;
import org.junit.jupiter.api.Test;

public class OnExceptionBeforeErrorHandlerIssueTest extends ContextTestSupport {

    @Test
    public void testOk() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(0);
        getMockEndpoint("mock:dead").expectedMessageCount(0);
        getMockEndpoint("mock:result").expectedMessageCount(1);

        template.sendBody("direct:start", "Hello World");

        assertMockEndpointsSatisfied();
    }

    @Test
    public void testKaboom() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(0);
        getMockEndpoint("mock:dead").expectedMessageCount(1);
        getMockEndpoint("mock:result").expectedMessageCount(0);

        template.sendBody("direct:start", "kaboom");

        assertMockEndpointsSatisfied();
    }

    @Test
    public void testIllegal() throws Exception {
        context.getRouteController().startRoute("foo");

        getMockEndpoint("mock:error").expectedMessageCount(1);
        getMockEndpoint("mock:dead").expectedMessageCount(0);
        getMockEndpoint("mock:result").expectedMessageCount(0);

        template.sendBody("direct:start", "illegal");

        assertMockEndpointsSatisfied();
    }

    @Override
    protected RouteBuilder createRouteBuilder() {
        return new RouteBuilder() {
            @Override
            public void configure() {
                onException(IllegalArgumentException.class).handled(true).setBody().constant("Handled").to("mock:error").end();

                // usually error handler should be defined first (before
                // onException),
                // but its not enforced
                errorHandler(deadLetterChannel("mock:dead").useOriginalMessage());

                from("direct:start").routeId("foo").noAutoStartup().process(new Processor() {
                    public void process(Exchange exchange) {
                        String body = exchange.getIn().getBody(String.class);
                        if ("illegal".equals(body)) {
                            throw new IllegalArgumentException("I cannot do this");
                        } else if ("kaboom".equals(body)) {
                            throw new RuntimeException("Kaboom");
                        }
                    }
                }).to("mock:result");
            }
        };
    }
}
